#!/usr/bin/env perl -w

# 20210201

use strict;
use warnings;

my @files = <DATA>;


foreach my $file (@files) {
    &make_seqfiles($file);
}

sub make_seqfiles {
  #my $gbkFile = "test_rna.gbk";
  #my $gbkFile = "S-purpuratus_GCF_000002235.4_Spur_4.2_rna.gbff";
  my ($gbkFile) = @_;
  
  chomp($gbkFile);
  print "$gbkFile\n";

  open(INFILE, $gbkFile) or die "$!";
  
  my $outFile_prot = "out_prot_".$gbkFile.".txt";
  my $outFile_nucl = "out_nucl_".$gbkFile.".txt";
  
  my $flag = 0;
  my $flag_product = 0;
  my $flag_translation = 0;
  my $flag_ORIGIN = 0;
  
  my $locus_id = "";
  my $protein_id = "";
  my $product= "";
  my $GeneID = "";
  my $seq_prot = "";
  my $seq_nucl = "";
  
  my $count = 0;
  open(PROT, ">$outFile_prot");
  open(NUCL, ">$outFile_nucl");
  while (my $line = <INFILE>){
    chomp($line);
    if ($line =~ /^LOCUS/){
        $locus_id = $line;
    }
    
    if ($flag == 1){
      if ($line =~ /protein_id=/) {
        #print "$line\n";
        $line =~ /protein_id="([^"]+)"/;
        $protein_id = $1;
        $protein_id =~ s/_//g;
      }
  
  
      if($flag_product == 1){
        if($line =~ /protein_id=/){
          $flag_product = 0;
          next;
        } else {
          $line =~ s/^ +/-/;
          $product .= $line;
          $flag_product = 0 if($line =~ /"/);
        }
      }       
      if ($line =~ /product=/){
        $flag_product = 1; 
        $product .= $line
      }
  
      if ($line =~ /GeneID:/){
        $line =~ /db_xref="GeneID:([^"]+)"/;
        $GeneID = $1;     
      }
  
      if ($flag_ORIGIN == 1) {
        $seq_nucl .= $line;
      }
      if ($line =~ /^ORIGIN/) {
        $flag_ORIGIN = 1;
      }
      if ($flag_translation == 1) {
        $seq_prot .= $line;
        $flag_translation = 0 if $line =~ /"/;
      }
      if ($line =~ /translation=/) {
        if($line =~ /translation="([^"]+)"/){
          $seq_prot = $line;
        } else {
          $seq_prot .= $line;
          $flag_translation = 1;
        }
      }
  
      if ($line =~ /^\/\//) {
        $flag = 0;
        $flag_ORIGIN = 0;
        
        if($protein_id){
          if($GeneID){
            $product =  &change_product($product);
            $seq_prot = &change_seq_prot($seq_prot);
            $seq_nucl = &change_seq_nucl($seq_nucl);
          
            print PROT ">$protein_id gene:$GeneID transcript:$protein_id description:$product \n";
            print PROT "$seq_prot\n";
    
            print NUCL ">$protein_id gene:$GeneID \n";
            print NUCL "$seq_nucl\n";
          }
        } else {
          print "No protID or geneID in $locus_id. Skipped.\n";
        }
  
        $locus_id = "";
        $protein_id = "";
        $product = "";
        $GeneID = "";
        $seq_prot = "";
        $seq_nucl = "";
      }
  
    }
  
    if ($line =~ /^ +CDS/){
      #print "$line\n";
      $flag = 1;
    }
  
    if ($line =~ /^\/\//){
      print "  $count\n" if $count % 1000 == 0;
      $count++ 
    }
  
    #last if $count == 100;
  }
  close INFILE;
  close PROT;
  close NUCL;
  print "\n";
}

sub change_seq_nucl{
  my ($seq_nuclSR) = @_;
  $seq_nuclSR =~ s/[ \d\/]//g;
  $seq_nuclSR = uc($seq_nuclSR);
  return($seq_nuclSR);

  
}

sub change_seq_prot{
  my ($seq_protSR) = @_;
  $seq_protSR =~ s/^.*translation="//;
  $seq_protSR =~ s/[ "]//g;
  return($seq_protSR);
}


sub change_product{
  my ($productSR) = @_; 
  $productSR =~ s/ +\/product="//g;
  $productSR =~ s/ +$//g;
  $productSR =~ s/"$//g;
  $productSR =~ s/[\/ ">]/-/g;
  $productSR =~ s/-{2,}/-/g;
  return($productSR);
}

exit;

__END__
test_rna.gbk
